@@ -11,7 +11,8 @@ import zipfile
 
 logger = logging.getLogger(__name__)
 
-rx_setuptools = re.compile(os.path.join('^.*', '(setuptools|distribute)-.*?-py\d.\d.egg$'))
+rx_setuptools = re.compile(
+    os.path.join('^.*', '(setuptools|distribute)-.*?-py\d.\d.egg$'))
 
 
 def copytree(src, dst, symlinks=0, allowed_basenames=None, exclude=[]):
@@ -283,6 +284,21 @@ class Recipe(zc.recipe.egg.Eggs):
             copytree(s, options['app-directory'], True,
                      exclude=options.get('exclude', '').split())
 
+    def patch_sdk(self, patch_options, patch_file):
+        """Patches the SDK's source tree."""
+        if patch_options:
+            patch_options = patch_options.split(' ')
+        else:
+            patch_options = '-p1'
+        gae = os.path.join(self.buildout['buildout']['parts-directory'],
+                           'google_appengine')
+        os.chdir(gae)
+        patch_cmd = ['patch'] + patch_options + ['<', patch_file]
+        retcode = os.system(' '.join(patch_cmd))
+        if retcode != 0:
+            raise Exception("patching the SDK failed")
+        return 0
+
     def install(self):
         """Creates the part."""
         options = self.options
@@ -305,6 +321,12 @@ class Recipe(zc.recipe.egg.Eggs):
             if os.path.isdir(temp_dir):
                 shutil.rmtree(temp_dir, True)
         self.copy_sources()
+        patch_file = options.get('patch')
+        patch_options = options.get('patch-options')
+        if patch_file:
+            if options.get('appengine-lib'):
+                raise Exception("patching preinstalled SDK not allowed")
+            self.patch_sdk(patch_options, patch_file)
         return ()
 
     def update(self):
@@ -313,4 +335,10 @@ class Recipe(zc.recipe.egg.Eggs):
         reqs, ws = self.working_set()
         self.setup_bin(ws)
         self.copy_sources()
+        patch_file = options.get('patch')
+        patch_options = options.get('patch_options')
+        if patch_file:
+            if options.get('appengine-lib'):
+                raise Exception("patching preinstalled SDK not allowed")
+            self.patch_sdk(patch_options, patch_file)
         return ()
